@@ -46,6 +46,13 @@ except (ImportError,):
     except (ImportError,):
         pass
 
+IMPORT_LIBVOIKKO_SUCCESSFUL = False
+try:
+    import libvoikko
+    IMPORT_LIBVOIKKO_SUCCESSFUL = True
+except (ImportError,):
+    pass
+
 # Maximum words that should be returned.
 # This should a rather big number in order not
 # to throw away useful matches. But making it very huge
@@ -68,6 +75,7 @@ class Dictionary:
         self.max_word_len = 0 # maximum length of words in this dictionary
         self.enchant_dict = None
         self.pyhunspell_object = None
+        self.voikko = None
         self.load_dictionary()
 
     def load_dictionary(self):
@@ -84,7 +92,7 @@ class Dictionary:
             # List of languages where accent insensitive matching makes sense:
             accent_languages = (
                 'af', 'ast', 'az', 'be', 'bg', 'br', 'bs', 'ca', 'cs', 'csb',
-                'cv', 'cy', 'da', 'de', 'dsb', 'el', 'en', 'es', 'eu', 'fo',
+                'cv', 'cy', 'da', 'de', 'dsb', 'el', 'en', 'es', 'eu', 'fi', 'fo',
                 'fr', 'fur', 'fy', 'ga', 'gd', 'gl', 'grc', 'gv', 'haw', 'hr',
                 'hsb', 'ht', 'hu', 'ia', 'is', 'it', 'kk', 'ku', 'ky', 'lb',
                 'ln', 'lv', 'mg', 'mi', 'mk', 'mn', 'mos', 'mt', 'nb', 'nds',
@@ -104,6 +112,12 @@ class Dictionary:
             if DEBUG_LEVEL > 1:
                 LOGGER.debug(
                     'max_word_len = %s\n', self.max_word_len)
+            if self.name.split('_')[0] == 'fi':
+                self.enchant_dict = None
+                self.pyhunspell_object = None
+                if IMPORT_LIBVOIKKO_SUCCESSFUL:
+                    self.voikko = libvoikko.Voikko('fi')
+                return
             if IMPORT_ENCHANT_SUCCESSFUL:
                 try:
                     self.enchant_dict = enchant.Dict(self.name)
@@ -131,6 +145,184 @@ class Dictionary:
                         self.name)
                     self.pyhunspell_object = None
 
+    def spellcheck_enchant(self, word):
+        '''
+        Spellcheck a word using enchant
+
+        :param word: The word to spellcheck
+        :type word: String
+        :return: True if spelling is correct, False if not or unknown
+        :rtype: Boolean
+        '''
+        if not self.enchant_dict:
+            return False
+        # enchant does the right thing for all languages, including
+        # Korean, if the input is a Unicode string in NFC.
+        return self.enchant_dict.check(unicodedata.normalize('NFC', word))
+
+    def spellcheck_pyhunspell(self, word):
+        '''
+        Spellcheck a word using pyhunspell
+
+        :param word: The word to spellcheck
+        :type word: String
+        :return: True if spelling is correct, False if not or unknown
+        :rtype: Boolean
+        '''
+        if not self.pyhunspell_object:
+            return False
+        # pyhunspell needs its input passed in dictionary encoding.
+        # and also returns in dictionary encoding.
+        return self.pyhunspell_object.spell(
+            unicodedata.normalize('NFC', word).encode(
+                self.encoding, 'replace'))
+
+    def spellcheck_voikko(self, word):
+        '''
+        Spellcheck a word using voikko
+
+        :param word: The word to spellcheck
+        :type word: String
+        :return: True if spelling is correct, False if not or unknown
+        :rtype: Boolean
+        '''
+        if not self.voikko:
+            return False
+        # voikko works correctly if the input is a Unicode string in NFC.
+        return self.voikko.spell(unicodedata.normalize('NFC', word))
+
+    def spellcheck(self, word):
+        '''
+        Spellcheck a word using enchant, pyhunspell, or voikko
+
+        :param word: The word to spellcheck
+        :type word: String
+        :return: True if spelling is correct, False if not or unknown
+        :rtype: Boolean
+
+        >>> d = Dictionary('en_US')
+        >>> d.spellcheck('winter')
+        True
+
+        >>> d.spellcheck('winxer')
+        False
+        '''
+        if self.enchant_dict:
+            return self.spellcheck_enchant(word)
+        if self.pyhunspell_object:
+            return self.spellcheck_pyhunspell(word)
+        if self.voikko:
+            return self.voikko.spell(word)
+        return False
+
+    def has_spellchecking(self):
+        '''
+        Returns wether this dictionary supports spellchecking or not
+
+        :return: True if this dictionary spports spellchecking, False if not
+        :rtype: Boolean
+
+        Examples:
+
+        >>> d = Dictionary('en_US')
+        >>> d.has_spellchecking()
+        True
+
+        >>> d = Dictionary('zh_CN')
+        >>> d.has_spellchecking()
+        False
+        '''
+        if self.enchant_dict or self.pyhunspell_object or self.voikko:
+            return True
+        return False
+
+    def spellcheck_suggest_enchant(self, word):
+        '''
+        Return spellchecking suggestions for word using enchant
+
+        :param word: The word to return spellchecking suggestions for
+        :type word: String
+        :return: List of spellchecking suggestions, possibly empty.
+        :rtype: List of strings
+        '''
+        if not word or not self.enchant_dict:
+            return []
+        # enchant does the right thing for all languages, including
+        # Korean, if the input is NFC. It takes Unicode strings and
+        # returns Unicode strings, no encoding and decoding is
+        # necessary, neither for Python2 nor for Python3.
+        return [
+            unicodedata.normalize(
+                itb_util.NORMALIZATION_FORM_INTERNAL, x)
+            for x in
+            self.enchant_dict.suggest(unicodedata.normalize('NFC', word))
+            ]
+
+    def spellcheck_suggest_pyhunspell(self, word):
+        '''
+        Return spellchecking suggestions for word using pyhunspell
+
+        :param word: The word to return spellchecking suggestions for
+        :type word: String
+        :return: List of spellchecking suggestions, possibly empty.
+        :rtype: List of strings
+        '''
+        if not word or not self.pyhunspell_object:
+            return []
+        # pyhunspell needs its input passed in dictionary encoding.
+        return [
+            unicodedata.normalize(
+                itb_util.NORMALIZATION_FORM_INTERNAL, x)
+            for x in
+            self.pyhunspell_object.suggest(
+                unicodedata.normalize('NFC', word).encode(
+                    self.encoding, 'replace'))
+            ]
+
+    def spellcheck_suggest_voikko(self, word):
+        '''
+        Return spellchecking suggestions for word using voikko
+
+        :param word: The word to return spellchecking suggestions for
+        :type word: String
+        :return: List of spellchecking suggestions, possibly empty.
+        :rtype: List of strings
+        '''
+        if not word:
+            return []
+        return [
+            unicodedata.normalize(
+                itb_util.NORMALIZATION_FORM_INTERNAL, x)
+            for x in
+            self.voikko.suggest(unicodedata.normalize('NFC', word))
+            ]
+
+    def spellcheck_suggest(self, word):
+        '''
+        Return spellchecking suggestions for word using enchant, pyhunspell or voikko
+
+        :param word: The word to return spellchecking suggestions for
+        :type word: String
+        :return: List of spellchecking suggestions, possibly empty.
+        :rtype: List of strings
+
+        Examples:
+
+        >>> d = Dictionary('en_US')
+        >>> d.spellcheck_suggest('kamel')
+        ['camel', 'Camel']
+
+        >>> d.spellcheck_suggest('')
+        []
+        '''
+        if self.enchant_dict:
+            return self.spellcheck_suggest_enchant(word)
+        if self.pyhunspell_object:
+            return self.spellcheck_suggest_pyhunspell(word)
+        if self.voikko:
+            return self.spellcheck_suggest_voikko(word)
+        return []
+
 class Hunspell:
     '''A class to suggest completions or corrections
     using a list of Hunspell dictionaries
@@ -203,6 +395,56 @@ class Hunspell:
             for dictionary in self._dictionaries:
                 LOGGER.debug('%s\n', dictionary.name)
 
+    def spellcheck(self, input_phrase):
+        '''
+        Checks if a string is likely to be spelled correctly checking
+        multiple dictionaries
+
+        :param input_phrase: A string to spellcheck
+        :type input_phrase: String
+        :return: True if it is more likely to be spelled correctly,
+                 False if it is more likely to be spelled incorrectly.
+                 In detail this means:
+                 True:
+                     - If it is a correctly spelled word in at least one of
+                       the dictionaries supporting spellchecking
+                     - None of the dictionaries support spellchecking
+                     - Contains spaces, spellchecking cannot work
+                 else False.
+        :rtype: Boolean
+
+        Examples:
+
+        >>> h = Hunspell(['en_US', 'de_DE', 'ja_JP'])
+        >>> h.spellcheck('Hello')
+        True
+
+        >>> h.spellcheck('Grüße')
+        True
+
+        >>> h.spellcheck('Gruße')
+        False
+
+        >>> h = Hunspell(['en_US', 'ja_JP'])
+        >>> h.spellcheck('Grüße')
+        False
+
+        >>> h = Hunspell(['ja_JP'])
+        >>> h.spellcheck('Grüße')
+        True
+        '''
+        if ' ' in input_phrase:
+            return True
+        spellchecking_dictionaries_available = False
+        spellcheck_total = False
+        for dictionary in self._dictionaries:
+            if dictionary.has_spellchecking():
+                spellchecking_dictionaries_available = True
+                spellcheck_total |= dictionary.spellcheck(input_phrase)
+        if not spellcheck_total and spellchecking_dictionaries_available:
+            return False
+        return True
+
     def suggest(self, input_phrase):
         # pylint: disable=line-too-long
         '''Return completions or corrections for the input phrase
@@ -234,9 +476,6 @@ class Hunspell:
         >>> h.suggest('Alpengluhen')[0]
         ('Alpenglühen', 0)
 
-        >>> h.suggest('filosofictejsi')
-        [('filosofičtější', 0), ('filosofičtěji', -1)]
-
         >>> h.suggest('filosofictejs')[0]
         ('filosofičtější', 0)
 
@@ -312,72 +551,31 @@ class Hunspell:
                             (x, 0)
                             for x in dictionary.words
                             if x.startswith(input_phrase)])
-                if dictionary.enchant_dict:
-                    if len(input_phrase) >= 4:
-                        # Always pass NFC to enchant and convert the
-                        # result back to the internal normalization
-                        # form (NFD) (enchant does the right thing for
-                        # Korean if the input is NFC).  enchant takes
-                        # unicode strings and returns unicode strings,
-                        # no encoding and decoding to and from the
-                        # hunspell dictionary encoding is necessary
-                        # (neither for Python2 nor Python3).
-                        # (pyhunspell needs to get its input passed
-                        # in dictionary encoding and also returns it
-                        # in dictionary encoding).
-                        if dictionary.enchant_dict.check(input_phrase_nfc):
-                            # This is a valid word in this dictionary.
-                            # It might have been missed by the matching
-                            # above because the dictionary might not
-                            # contain all possible word forms (The
-                            # prefix and suffix information has been
-                            # ignored). But hunspell knows about this,
-                            # if hunspell thinks it is a correct word,
-                            # it must be counted as a match of course:
-                            suggested_words[input_phrase] = 0
-                        extra_suggestions = [
-                            unicodedata.normalize(
-                                itb_util.NORMALIZATION_FORM_INTERNAL, x)
-                            for x in
-                            dictionary.enchant_dict.suggest(input_phrase_nfc)
-                        ]
-                        suggested_words.update([
-                            (suggestion, -1)
-                            for suggestion in extra_suggestions
-                            if suggestion not in suggested_words])
-                elif dictionary.pyhunspell_object:
-                    if len(input_phrase) >= 4:
-                        # Always pass NFC to pyhunspell and convert
-                        # the result back to the internal
-                        # normalization form (NFD) (hunspell does the
-                        # right thing for Korean if the input is NFC).
-                        if dictionary.pyhunspell_object.spell(
-                                input_phrase_nfc.encode(
-                                    dictionary.encoding, 'replace')):
-                            # This is a valid word in this dictionary.
-                            # It might have been missed by the matching
-                            # above because the dictionary might not
-                            # contain all possible word forms (The
-                            # prefix and suffix information has been
-                            # ignored). But hunspell knows about this,
-                            # if hunspell thinks it is a correct word,
-                            # it must be counted as a match of course:
-                            suggested_words[input_phrase] = 0
-                        extra_suggestions = [
-                            unicodedata.normalize(
-                                itb_util.NORMALIZATION_FORM_INTERNAL, x)
-                            for x in
-                            dictionary.pyhunspell_object.suggest(
-                                input_phrase_nfc.encode(
-                                    dictionary.encoding, 'replace'))
-                        ]
-                        suggested_words.update([
-                            (suggestion, -1)
-                            for suggestion in extra_suggestions
-                            if suggestion not in suggested_words])
+                if len(input_phrase) >= 4:
+                    if dictionary.spellcheck(input_phrase):
+                        # This is a valid word in this dictionary.
+                        # It might have been missed by the
+                        # matching above because the dictionary
+                        # might not contain all possible word
+                        # forms (The prefix and suffix information
+                        # has been ignored). But the spell checker
+                        # knows about this, if the spell checker
+                        # thinks it is a correct word, it must be
+                        # counted as a match of course:
+                        suggested_words[input_phrase] = 0
+                    extra_suggestions = [
+                        unicodedata.normalize(
+                            itb_util.NORMALIZATION_FORM_INTERNAL, x)
+                        for x in
+                        dictionary.spellcheck_suggest(input_phrase)
+                    ]
+                    suggested_words.update([
+                        (suggestion, -1)
+                        for suggestion in extra_suggestions
+                        if suggestion not in suggested_words])
             else:
                 if (dictionary.name[:2]
-                        not in ('ja', 'ja_JP',
+                        not in ('fi', 'ja', 'ja_JP',
                                 'zh', 'zh_CN', 'zh_TW', 'zh_MO', 'zh_SG')):
                     # For some languages, hunspell dictionaries don’t
                     # exist because hunspell makes no sense for these
